@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Carp;
 
-# $Revision: 517 $ - $Date: 2006-08-22T17:32:37.165669Z $
+# $Revision: 919 $ - $Date: 2007-01-20T23:04:07.761860Z $
 
 =pod 
 
@@ -134,17 +134,19 @@ sub interpolation {
 sub interpolating {
     my $self=shift;
 
-    $self->{interpolating}=shift if (@_);
+    if ( @_ ) {
+        $self->processor->set_interpolation( 
+            $self->{interpolating} = shift
+        );
+    }
 
-    return exists $self->{interpolating} ?
-			$self->{interpolating} :
-		    !$XML::XPathScript::DoNotInterpolate; # Obsolete, for compatibility:
+    return $self->{interpolating} || 0;
 }
 
 =head2 interpolation_regex
 
-    $regex = $XML::XPathScript::curent->interpolation_regex
-    $XML::XPathScript::curent->interpolation_regex( $regex )
+    $regex = $XML::XPathScript::current->interpolation_regex
+    $XML::XPathScript::current->interpolation_regex( $regex )
 
 Gets or sets the regex to use for interpolation. The value to be 
 interpolated must be capture by $1. 
@@ -162,7 +164,11 @@ Example:
 sub interpolation_regex {
     my $self = shift;
 
-    $self->{interpolation_regex} = shift if @_;
+    if ( my $regex = shift ) {
+        $self->processor->set_interpolation_regex( 
+            $self->{interpolation_regex} = $regex
+        )
+    }
 
     return $self->{interpolation_regex};
 }
@@ -182,6 +188,7 @@ Unicode mess>.
 sub binmode {
     my ($self)=@_;
     $self->{binmode}=1;
+    $self->{processor}->enable_binmode;
     binmode ORIGINAL_STDOUT if (! defined $self->{printer});
     return;
 }
@@ -225,14 +232,14 @@ preprocess time.
 
 =cut "
 
-use vars qw( $XML_parser $DoNotInterpolate $debug_level );
+use vars qw( $XML_parser $debug_level );
 
 use Symbol;
 use File::Basename;
 use XML::XPathScript::Processor;
 use XML::XPathScript::Template;
 
-our $VERSION = '1.46';
+our $VERSION = '1.47';
 
 $XML_parser = 'XML::LibXML';
 
@@ -251,9 +258,6 @@ die "parser $XML_parser unknown\n" unless $use_parser{$XML_parser};
 eval $use_parser{$XML_parser}.";1" 
     or die "couldn't import $XML_parser";
 
-# By default, we interpolate
-$DoNotInterpolate = 0;
-
 # internal variable for debugging information. 
 # 0 is total silence and 10 is complete verbiage
 $debug_level = 0;
@@ -331,20 +335,26 @@ sub new {
     my %params = @_;
     my $self = \%params;
     bless $self, $class;
+    $self->{processor} = XML::XPathScript::Processor->new;
     $self->set_xml( $params{xml} ) if $params{xml};
-	$self->{interpolation_regex} ||= qr/{(.*?)}/;
+
+    $self->interpolation( exists $params{interpolation} 
+                               ? $params{interpolation} : 1 );
+
+    $self->interpolation_regex( $params{interpolation_regex} 
+                                || qr/{(.*?)}/ );
+
+
 
     if (  $XML::XPathScript::XML_parser eq 'XML::XPath' ) {
-        eval <<END_EVAL;
-			use XML::XPath 1.0;
-			use XML::XPath::XMLParser;
-			use XML::XPath::Node;
-			use XML::XPath::NodeSet;
-			use XML::Parser;
-END_EVAL
+        require XML::XPath;
+        require XML::XPath::XMLParser;
+        require XML::XPath::Node;
+        require XML::XPath::NodeSet;
+        require XML::Parser;
     } 
     else {
-        eval 'use XML::LibXML';
+        require XML::LibXML;
     }
 
     croak $@ if $@;
@@ -401,6 +411,23 @@ sub transform {
     return $output;
 }
 
+=head2 set_dom
+
+    $xps->set_dom( $dom )
+
+Set the DOM of the document to process. I<$dom>
+must be a node object of one of the supported 
+parsers (XML::LibXML, XML::XPath, B::XPath).
+
+=cut
+
+sub set_dom {
+    my( $self, $dom ) = @_;
+    $self->{dom} = $dom;
+    $self->{processor}->set_dom( $dom );
+    return $self;
+}
+
 =head2 set_xml
 
     $xps->set_xml( $xml )
@@ -415,7 +442,15 @@ sub set_xml {
 
     $self->{xml} = $xml;
 
-    return ref $xml ? $self->_set_xml_ref() : $self->_set_xml_scalar();
+    my $retval = ref $xml ? $self->_set_xml_ref() 
+                          : $self->_set_xml_scalar()
+                          ;
+
+    $self->{processor}->set_dom( $self->{dom} );
+    
+    return $retval;
+
+    # FIXME
 
 my $xpath;
 
@@ -602,7 +637,8 @@ sub process {
 
     croak "xml document not defined" unless $self->{dom};
 
-	$self->{dom}->ownerDocument->setEncoding( "UTF-8" ) 
+    # FIXME
+	eval { $self->{dom}->ownerDocument->setEncoding( "UTF-8" ) }
 		if $XML_parser eq 'XML::LibXML';
 
 	{
@@ -662,6 +698,94 @@ sub extract {
 
     my $contents = $self->read_stylesheet( $stylesheet );
 
+    my @tokens = split /(<%[-=~#@]*|-?%>)/, $contents;
+
+    no warnings qw/ uninitialized /;
+
+    my $script;
+    my $line = 1;
+    TOKEN:
+    while ( @tokens ) {
+        my $token = shift @tokens;
+
+        if ( -1 == index $token, '<%' ) {
+            $line += $token =~ tr/\n//;
+            $token =~ s/\s+$// if  -1 < index $tokens[0], '<%'
+                               and -1 < index $tokens[0], '-';
+            $token =~ s/\|/\\\|/g;
+            # check for include
+            $token =~ s{<!--#include.+file=(['"])(.*?)\1.*?-->}
+                       { '|);'
+                         . $self->include_file( $2, @includestack)
+                         . 'print(q|'}seg;
+            $script .= 'print(q|'.$token.'|);' if length $token;
+
+            next TOKEN;
+        }
+
+        $script .= "\n#line $line $filename\n";
+
+        my $opening_tag = $token;
+        my $code;
+        my $closing_tag;
+        my $level = 1;
+        while( @tokens ) {
+            my $t = shift @tokens;
+            $level++ if -1 < index $t, '<%';
+            $level-- if -1 < index $t, '%>';
+            if ( $level == 0 ) {
+                $closing_tag = $t;
+                last;
+            }
+            $code .= $t;
+        }
+
+        die "stylesheet <% %>s are unbalanced: $opening_tag$code\n"
+            unless $closing_tag;
+
+        $line += $code =~ tr/\n//;
+
+        if ( -1 < index $opening_tag, '=' ) {
+            $script .= 'print( '.$code.' );';
+        }
+        elsif ( -1 < index $opening_tag, '~' ) {
+            $code =~ s/^\s+//; 
+            $code =~ s/\s+$//; 
+            $script .= 'print $processor->apply_templates( qq<'. $code .'> );';
+        }
+        elsif( -1 < index $opening_tag, '#' ) {
+            # do nothing
+        }
+        elsif( -1 < index $opening_tag, '@' ) {
+            $code =~ s/^\s+(\S+).*?\n//;    # strip first line
+            my $tag = $1 
+                or die "tag name missing in <%\@ %> at line $line\n";
+
+            my $here_delimiter = 'END_TAG';
+            while ( $code =~ /$here_delimiter/ ) {
+                $here_delimiter .= 'x';
+            }
+            $script .= <<END_SNIPPET;
+\$template->set( $tag => { content => <<'$here_delimiter' } );
+$code
+$here_delimiter
+END_SNIPPET
+        }
+        else {
+                    # always add a ';', just in case
+            $script .= $code . ';';
+        }
+
+        if ( -1 < index $closing_tag, '-' ) {
+            $tokens[0] =~ s/^\s*//;
+            my $temp = $&;
+            $line += $temp =~ tr/\n//;
+        }
+    }
+
+    return $script;
+
+    # FIXME not needed anymore
     # <%- -%> magic
     $contents =~ s#(\s+)<%-([=~]?)#<%$2$1#gs;
     $contents =~ s#-%>(\s+)#$1%>#gs;
@@ -669,8 +793,8 @@ sub extract {
     # <%~ %> magic
     $contents =~ s#<%~\s+(\S+)\s+%>#<%= apply_templates( qq<$1> ) %>#gs;
 
-    my $script="#line 1 $filename\n",
-    my $line = 1;
+    $script="#line 1 $filename\n",
+    $line = 1;
 
     while ($contents =~ /\G(.*?)(<!--#include|<%[=#]?)/gcs) {
         my ($text, $type) = ($1, $2);
@@ -811,9 +935,9 @@ sub include_file {
 
 =pod "
 
-=item I<compile()>
+=head2 I<compile()>
 
-=item I<compile(varname1, varname2,...)>
+=head2 I<compile(varname1, varname2,...)>
 
 Compiles the stylesheet set at I<new()> time and returns an anonymous
 CODE reference. 
@@ -868,6 +992,13 @@ sub compile {
     my $package=gen_package_name();
 
 	my $extravars = join ',', @extravars;
+
+    my $processor = $self->{processor};
+
+    # needs to be eval'ed first for the constants
+    # to be seen
+    eval "package $package;"
+        ."\$processor->import_functional();";
 	
 	my $eval = <<EOT;
 		    package $package;
@@ -875,17 +1006,21 @@ sub compile {
 		    no warnings; # written stylesheets
 			
 			use $XML_parser;  
-		    BEGIN {XML::XPathScript::Processor->import;}
+
 		    sub {
 		    	my (\$self, $extravars ) = \@_;
+                my \$processor = processor();
 				local \$XML::XPathScript::current=\$self;
-		    	my \$t = \$XML::XPathScript::current->{t} 
+		    	my \$t = \$processor->{template} 
                             = XML::XPathScript::Template->new();
                 my \$template = \$t;
-				local \$XML::XPathScript::trans = \$t; # Yes,
-				# this does the sharing! Perl is a bizarre and
-				# wonderful language.
-				local \$XML::XPathScript::xp=\$self->{dom};
+                local \$XML::XPathScript::trans = \$t;
+                #\$processor->{doc} = \$self->{dom};
+                #\$processor->{parser} = '$XML_parser';
+                #\$processor->{binmode} = \$self->{binmode};
+                #\$processor->{is_interpolating} = \$self->interpolation;
+                #\$processor->{interpolation_regex} = \$self->interpolation_regex;
+
 				$script
 		    }
 EOT
@@ -953,6 +1088,17 @@ sub get_stylesheet_dependencies {
     return sort keys %{$self->{stylesheet_cache}};
 }
 
+=head2 processor
+
+    $processor = $xps->processor
+
+Returns the processor object associated with I<$xps>.
+
+=cut
+
+sub processor {
+    return $_[0]->{processor};
+}
 
 =head1 FUNCTIONS
 
@@ -1069,10 +1215,17 @@ http://rt.cpan.org/Public/Dist/Display.html?Name=XML-XPathScript .
 
 Current maintainers: 
 Yanick Champoux <yanick@cpan.org> 
-and Dominique Quatravaux <dom@idealx.com>
+and Dominique Quatravaux <domq@cpan.org>
 
 Created by Matt Sergeant <matt@sergeant.org>
 
+=head1 THANKS
+
+Thanks to Tim Nelson for pretty nifty suggestions and 
+patches. We sure hope the new B<insteadofchildren>
+tag will make XSL users flock to XPS like ants to 
+a melting chocolate bunny, as he promised. ;-)
+
 =head1 LICENSE
 
 This is free software. You may distribute it under the same terms as
